from nanugo import nanugo
from nanugo.log import logger
import os, logging
from PIL import Image

logger.setLevel(logging.DEBUG)
get_file_name = lambda path: os.path.basename(path)

def test_default_deck():
    builder = nanugo.Builder()
    errors = []
    deck = builder.build_deck("test", "test/test.pdf")
    if file_name := get_file_name(deck.media_list[0]) != "test_p0_q.jpeg":
        errors.append(f"File name doesn't seem right: {file_name}")
    with Image.open(deck.media_list[0]) as test_subj_image:
        if test_subj_image.size != (540, 390):
            errors.append(f"Split page size doesn't seem right: {test_subj_image.size}")
    if media_len := len(deck.media_list) != 2:
        errors.append(f"len(media_list) is not 2: {media_len}")

    assert not errors, "Errors occured: \n{}".format("\n".join(errors))


def test_weird_deck():
    builder = nanugo.Builder()
    errors = []
    deck = builder.build_deck("test", "test/test.pdf", vertical=True, ratio=(0.4, 0.6))
    if file_name := get_file_name(deck.media_list[0]) != "test_p0_q.jpeg":
        errors.append(f"File name doesn't seem right: {file_name}")
    with Image.open(deck.media_list[0]) as test_subj_image:
        if test_subj_image.size != (216, 780):
            errors.append(f"Split page size doesn't seem right: {test_subj_image.size}")
    if media_len := len(deck.media_list) != 2:
        errors.append(f"len(media_list) is not 2: {media_len}")

    assert not errors, "Errors occured: \n{}".format("\n".join(errors))

if __name__ == "__main__":
    test_default_deck()
    test_weird_deck()


from typing import List, Dict, Tuple
import pypdfium2 as pdfium
import PIL, logging, warnings, os
from ..log import logger


def convert_pdf(pdf_path: str) -> List[Dict[int, PIL.Image.Image]]:
    '''Converts PDF files to PIL.Image.Image object.

    Args:
        pdf_path (str): Path for the pdf file to be converted

    Returns:
        List[Dict[int, PIL.Image.Image]]: List of PIL.Image.Image objects.
    '''
    result = []

    pdf_file = pdfium.PdfDocument(pdf_path)
    page_indicies = [i for i in range(len(pdf_file))]

    # If your code shall be frozen into an executable, multiprocessing.freeze_support() needs to be called at the start of the if __name__ == "__main__": block if using this method.
    renderer = pdf_file.render(pdfium.PdfBitmap.to_pil, page_indices=page_indicies)

    for image in renderer:
        result.append(image)

    logger.debug(
        f"{len(result)} pages has been successfully rendered: {os.path.split(pdf_path)[-1]}"
    )

    return result


def split_page(
    image: PIL.Image.Image, vertical: bool = False, ratio: tuple = (0.5, 0.5)
) -> Tuple[PIL.Image.Image, PIL.Image.Image]:
    '''Splits given PIL.Image.Image object into two.

    Args:
        image (PIL.Image.Image): PIL.Image.Image object to be split
        vertical (bool, optional): if true, the image will be split vertically, not horizontally. Defaults to False.
        ratio (tuple, optional): ratio to which the image will be split. Defaults to (0.5, 0.5).

    Returns:
        Tuple[PIL.Image.Image, PIL.Image.Image]: A tuple of two PIL.Image.Image objects. First Image will going to be the top/left one by default.
    '''
    d_width, d_height = image.size  # Get default sizes

    if sum(ratio) != 1:
        warnings.warn(
            "Custom ratio was given but the sum of it is not 1. If this is intended, you can safely ignore this warning.",
            UserWarning,
        )

    if vertical:
        ratio_l, ratio_r = ratio
        image_l = image.crop((0, 0, d_width * ratio_l, d_height))
        image_r = image.crop((d_width * (1 - ratio_r), 0, d_width, d_height))

        return (image_l, image_r)

    else:
        ratio_t, ratio_b = ratio
        image_t = image.crop((0, 0, d_width, d_height * ratio_t))
        image_b = image.crop((0, d_height * (1 - ratio_b), d_width, d_height))

        return (image_t, image_b)



from glob import glob
from .tools import validations
from .log import logger
from .nanugo import Builder
from setuptools_scm import get_version
import argparse, sys, os, logging


def main(args=None):
    parsed_args = get_args(args)

    if parsed_args.debug:
        logger.setLevel(logging.DEBUG)

    logger.debug(parsed_args)

    pkg_name = parsed_args.package_name[0]
    export_file_path = os.path.join(parsed_args.dest, f"{pkg_name}.apkg")

    validations.file_exists_prompt(export_file_path, pkg_name)

    file_paths = []

    for arg in parsed_args.pdf_files:
        try:
            validations.pdf_file_arg(arg)
            file_paths += glob(arg)
        except:
            sys.exit()

    logger.info(f"Total {len(file_paths)} file(s) will be converted.")

    builder = Builder()
    decks = []

    file_name_only = lambda path: os.path.splitext(os.path.basename(path))[0]

    for file in file_paths:
        deck = builder.build_deck(
            file_name_only(file),
            file,
            ratio=tuple(parsed_args.ratio),
            vertical=parsed_args.vertical,
            inversed=parsed_args.inversed,
        )
        decks.append(deck)

    pkg = builder.build_pkg(decks)
    pkg.write_to_file(export_file_path)

    logger.info(f"Conversion completed: {export_file_path}")

    # No need for explicit return
    # return pkg


def get_args(args):
    parser = get_parser()
    return parser.parse_args(args)


def get_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        allow_abbrev=True,
        description="Turn your handwritten pdf sheets to Anki deck(s).",
        epilog="by @lewisleedev",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "package_name",
        action="store",
        type=str,
        nargs=1,
        help="File name for your .apkg file. Do not put file extension in the end.",
    )
    parser.add_argument(
        "pdf_files",
        help="Input PDF file to convert",
        nargs="*",
    )
    parser.add_argument(
        "-d,",
        "--dest",
        dest="dest",
        metavar="",
        action="store",
        type=validations.dir_path,
        default="./",
        help="Destination folder for your .apkg output. Defaults to current directory.",
    )
    parser.add_argument(
        "--ratio",
        dest="ratio",
        metavar="",
        action="store",
        type=float,
        nargs=2,
        default=(0.5, 0.5),
        help="Custom ratio to which each pages will be split. Defaults to (0.5 0.5). It's best to keep the sum of the ratios to 1. First number is the ratio for the top or left.",
    )
    parser.add_argument(
        "--inversed",
        "-i",
        dest="inversed",
        action="store_true",
        help="Inverses the Question/Answer side.",
    )
    parser.add_argument(
        "--vertical",
        "-v",
        dest="vertical",
        action="store_true",
        help="Splits vertically instead of horizontally",
    )
    parser.add_argument(
        "--debug",
        "--d",
        dest="debug",
        action="store_true",
        help="For even bigger nerds",
    )
    parser.add_argument(
        "--version",
        "-V",
        action="version",
        version=get_version(),
        help="Shows version and exits.",
    )

    return parser


if __name__ == "__main__":
    main()
